/*****************************************************************************
**
**  File Name:          Megaload.c
**  Author:             Dave Helland
**  Date:               5/21/04
**
**  Purpose/Description:
**       1. Upload HEX files via serial port to program FLASH in
**            Atmel AMR microprocessors running :  Bootload.c
**       2. Verify HEX files via serial port are same as FLASH
**       3. Upload HEX files via serial port to program EEPROM
**       4. Verify HEX files via serial port are same as EEPROM
**       5. Upload TEXT files via serial port
**       6. Function as VT100 Terminal Emulator
**       7. Log Received data to file
**       8. Download config from EEPROM to file
**       9. Download tables from EEPROM to file
**
**  CSC:  MEGALOAD
**
**  Public Data:
**
**  Private Data:
**
**  Functions:
**
**  Version/Revision History:
**
**
**  Version ID   Date      Author        SPCR No.    Description
**
**  Atmel MegasquirtAVR download program
**  0.0    3/21/01   DRH   great portions borrowed PULOAD
**  1.0    5/04/04   DRH   Initial Release
**  1.1   25/08/05   DB    Added download of config and tables to file
**
**
***************************************************************************/
/**************************************************************************
* Usage: megaloader [fileword] [flagword] [speedword] [port]
* [fileword]
*    [filename.ext] optional - upload Hex flash file
*    [-d filename.ext] optional - upload text file via emulator
*    [-e filename.ext] optional - upload Hex eeprom file (N/A)
*    [-l filename.ext] optional - log file for terminal emulator
*    [-o filename.ext] optional - download config to file
*    [-a filename.ext] optional - download tables to file
* [flagword] optional (case sensitive)
*    [-w] : write flash (otherwise only verify). This implies 'E'
*    [-t] : talk to msavr (application) first,
*           issuing the menucommand to drive it to bootloader
*    [-E] : exit bootloader at the end (try to start msavr app.)
*    [-T] : Time Tag received terminal emulator messages on CRLF
*    [-V] : Verbose messages
*    [-v] : MEGALOADER version number
* [speedword] optional
*    [-r 19200, 38400, 57600, 115200] optional -
*           (use after -t) download baud rate (default 19200)
*    [-b rate]   optional - terminal emulator baud rate (default 9600)
* [port] optional
*    [-c COM1], [-c /dev/ttyS0]\r\n
*
***************************************************************************/
/**************************************************************************
* Terminal Mode Commands:
*   Commands are single control characters.
*      ^A (control-a) : Toggle display mode
*          Display Modes are:
*             VT100 mode : interpret VT100 screen control sequences
*             ascii hex :  dumb terminal with control chars printed as <hex>
*             hex : all chars printed as 2 char hex bytes
*      ^B (control-b) :  clear the screen
*      ^C (control-c) : Control-C Exit
*      ^F (control-f) : Prompt for text file to send
*      ^D (control-d) : Send text file
*      ^R (control-r) : Toggle RTS high (can be used for remote reset)
*      escape : Normal program exit
*
***************************************************************************/
/*
;*
;*	The following commands are from Atmel app note #109 with additions
;*      from megasquirtAVR.  Not all are supported by this program.
;*      Only the commands required by the MegasquirtAVR are supported.
;*      All commands start with a single letter. The programmer returns 13d
;*      (carriage return) or the data read after the command is finished.
;*
;*                                     +-------------+------------+------+
;*  Commands                           | Host writes | Host reads |      |
;*  --------                           +-----+-------+------+-----+      |
;*                                     | ID  | data  | data |     | Note |
;* +-----------------------------------+-----+-------+------+-----+------+
;* | Enter programming mode            | 'P' |       |      | 13d |   1  |
;* | Report autoincrement address      | 'a' |       | 	    | 'Y' |      |
;* | Set address                       | 'A' | ah al |      | 13d |   2  |
;* | Write program memory, low byte    | 'c' |    dd |      | 13d |   3  |
;* | Write program memory, high byte   | 'C' |    dd |      | 13d |   3  |
;* | Issue Page Write                  | 'm' |       |      | 13d |      |
;* | Read program memory               | 'R' |       |dd(dd)|     |   4  |
;* | Write data memory                 | 'D' |    dd |      | 13d |      |
;* | Read data memory                  | 'd' |       |   dd |     |      |
;* | Chip erase                        | 'e' |       |      | 13d |      |
;* | Write lock bits                   | 'l' |    dd |      | 13d |      |
;* | Write fuse bits                   | 'f' |    dd |      | 13d |  11  |
;* | Read fuse and lock bits           | 'F' |       |   dd |     |  11  |
;* | Leave programming mode            | 'L' |       |      | 13d |   5  |
;* | Select device type                | 'T' |    dd |      | 13d |   6  |
;* | Read signature bytes              | 's' |       | 3*dd |     |      |
;* | Return supported device codes     | 't' |       | n*dd | 00d |   7  |
;* | Return software identifier        | 'S' |       | s[7] |     |   8  |
;* | Return sofware version            | 'V' |       |dd dd |     |   9  |
;* | Return hardware version           | 'v' |       |dd dd |     |   9  |
;* | Return programmer type            | 'p' |       |   dd |     |  10  |
;* | Set LED                           | 'x' |    dd |      | 13d |  12  |
;* | Clear LED                         | 'y' |    dd |      | 13d |  12  |
;* | Universial command                | ':' |  3*dd |   dd | 13d |      |
;* | New universal command	       | '.' |  4*dd |   dd | 13d |      |
;* | Special test command	       | 'Z' |  2*dd |   dd |     |      |
;* | ultrafast buffer load after 'A'   | 'D' | kk buf|      | 13d |      |
;* | Load checksum type after 'D'      | 'k' |       |      |     |      |
;* | Verify checksum type after 'D'    | 'l' |       |      |     |      |
;* | store Marker Word at last mem loc | 'E' |       |      | 13d |      |
;* +-----------------------------------+-----+-------+------+-----+------+
;*
;* NOTE 1
;*	The Enter programming mode command MUST be sent one time prior to
;*	the other commands, with the exception of the 't', 'S', 'V', 'v'
;*	and 'T' commands. The 'T' command must be sent before this command
;*	(see note 6).
;*
;*	For programmers supporting both parallel and serial programming
;*	mode this command enters parallel programming mode. For programmers
;*	supporting only serial programming mode, this command enters serial
;*	programming mode.
;*
;* NOTE 2
;*	The ah and al are the high and low order bytes of the address. For
;*	parallel programmers this command issues the Load Address Low/High
;*	Byte command. For serial programmers the address byte is stored for
;*	use by the Read/Write commands.
;*
;* NOTE 3
;*	For parallel programmers this command issues the Program Flash
;*	command. For serial programmers this command iussues the Write
;*	Program Memory Command. For devices with byte-wide program memories
;*	only the low byte command should be used.
;*
;* NOTE 4
;*	The contents of the program memory at the address given by the 'A'
;*	command are written to the serial port in binary form. For byte
;*	wide memories one byte is written. For 16 bit memories two bytes
;*	are written,MSB first.
;*
;* NOTE 5
;*	This command must be executed after the programming is finished.
;*
;* NOTE 6
;*	The select device type command must be sent before the enter
;*	programming command
;*
;* NOTE 7
;*	The supported device codes are returned in binary form terminated
;*	by 0x00.
;*
;* NOTE 8
;*	This return a 7 character ASCII string identifying the programmer.
;*	For the development board it is "AVR DEV", for the parallel
;*	programmer it is "AVR PPR" and for the in-curcuit programmer it is
;*	"AVR ICP".
;*
;* NOTE 9
;*	The software/hardware version are returned as two ASCII numbers.
;*
;* NOTE 10
;*	This command should be used to identify the programmer type. The
;*	return value is 'S' for serial (or SPI) programmers or 'P' for
;*	parallel programmers.
;*
;* NOTE 11
;*	The write fuse bits command are available only on parallel
;*	programmers and only for AVR devices (device code < 0x80). The host
;*	should use the return programmer type command to determine the
;*	programmer type, do not use the  "AVR PPR" idenifier because other
;*	programmers may be available in the future.
;*
;* NOTE 12
;*	Currently only the AVR development board has LEDs. The other boards
;*	must implement this commands as NOPs.
;*
;* NOTE 13
;*      Devices using Page Mode Programming write one page of flash memory
;*      before issuing a Page Mode Write Pulse.
;*
;*
;*  Atmel AVRProg device_codes:
;*
;AT90S1200 rev. A    = 0x10,0
;AT90S1200 rev. B    = 0x11,0
;AT90S1200 rev. C    = 0x12,0
;AT90S1200           = 0x13,0
;AT90S2313           = 0x20,0
;AT90S4414           = 0x28,0
;AT90S4433           = 0x30,0
;AT90S2333           = 0x34,0
;AT90S8515           = 0x38,0
;ATmega8515          = 0x3A,'P'
;ATmega8515 BOOT     = 0x3B,'P'
;ATmega103           = 0x41,'P'
;ATmega603           = 0x42,'P'
;ATmega128           = 0x43,'P'
;ATmega128 BOOT      = 0x44,'P'
;AT90S2323           = 0x48,0
;AT90S2343           = 0x4C,0
;ATtiny11            = 0x50,0
;ATtiny10            = 0x51,0
;ATtiny12            = 0x55,0
;ATtiny15            = 0x56,0
;ATtiny19            = 0x58,0
;ATtiny28            = 0x5C,0
;ATtiny26            = 0x5E,'P'
;ATmega161           = 0x60,'P'
;ATmega161 BOOT      = 0x61,'P'
;ATmega163           = 0x64,'P'
;ATmega83            = 0x65,'P'
;ATmega163 BOOT      = 0x66,'P'
;ATmega83 BOOT       = 0x67,'P'
;AT90S8535           = 0x68,0
;ATmega8535??        = 0x6A,'P'
;ATmega8535 BOOT??   = 0x6B,'P'
;AT90S4434           = 0x6C,0
;AT90C8534           = 0x70,0
;AT90C8544           = 0x71,0
;ATmega32            = 0x72,'P'
;ATmega32 BOOT       = 0x73,'P'
;ATmega16            = 0x74,'P'
;ATmega16 BOOT       = 0x75,'P'
;ATmega8             = 0x76,'P'
;ATmega8 BOOT        = 0x77,'P'
;AT89C1051           = 0x80,0
;AT89C2051           = 0x81,0
;AT89S8252           = 0x86,0
;AT89S53             = 0x87,0
*/
/*********************************************************/
/*                   I N C L U D E                       */
/*********************************************************/

#include <ctype.h>
#include <stdio.h>

#ifdef __WIN32__
#include <conio.h>
#include <windows.h>
#endif

#ifdef  __linux__
#include <sys/time.h>
#include <curses.h>
#include "global.h"
#include "linux_screen.h"
#endif


/*********************************************************/
/*           P R O T O T Y P E                           */
/*********************************************************/
#define MAIN 1

#include "defines.h"
#include "main.h"
#include "hex_prog.h"
#include "interface.h"
#include "linux_serial.h"
/*********************************************************/

/*********************************************************/
/*           G L O B A L    V A R I A B L E S            */
/*********************************************************/

//extern int eep_rom;
//struct time t;
//struct date d;
FILE *fopen();
FILE *outfile, *in;
int log_file_open;
char ldr_filename[80];          /*filename to serial send */
char eep_filename[80];          /*filename to serial send */
char *ldr_file;                 /*filename to serial send */
char sCommPortName[10];         /*name of serial port */
char sBaudName[10];             /*speed of terminal emulator serial port */
char sDnBaudName[10];           /*speed of down load serial port */
char sArgName[80];              /*arg string */
//int debug=0;
int hex_mode;
int bin_mode;
int do_tty;
int write_flash;
int write_marker = 0;
int talk;
int skip_download;
int skip_hexload, skip_eepload;
int upload_c, upload_t;
int time_tag;
int tag_next;
int baud_rate;
int dnload_baud_rate = 19200;
//static HANDLE hStdout ;
//static HANDLE hStdin;
//static HANDLE hTimer ;
//DWORD dwDesiredAccess;
P_MEM mem_buf;

int did_flash = 0;
int serial_timeout = SERIAL_DELAY;
int verbose = 0;

/*********************************************************/
/*                   C O D E                             */
/*********************************************************/
void
usage(void)
{
  CleanUp();
  CloseConsole();

  printf("Usage: megaloader [fileword] [flagword] [speedword] [port]\r\n");
  printf("   [fileword] \r\n");
  printf("       [filename.ext] optional - download Hex flash file\r\n");
  printf
    ("       [-d filename.ext] optional - download text file via emulator\r\n");
  printf
    ("       [-e filename.ext] optional - download Hex eeprom file (N/A)\r\n");
  printf
    ("       [-l filename.ext] optional - log file for terminal emulator\r\n");
  printf
    ("       [-o filename.ext] optional - download config to file\r\n");
  printf
    ("       [-a filename.ext] optional - download tables to file\r\n");
  printf("   [flagword] optional (case sensitive)\r\n");
  printf
    ("       [-w] : write flash (otherwise only verify). This implies 'E'\r\n");
  printf("       [-t] : talk to msavr (application) first, \r\n");
  printf
    ("                 issuing the menucommand to drive it to bootloader\r\n");
  printf
    ("       [-E] : exit bootloader at the end (try to start msavr app.)\r\n");
  printf
    ("       [-T] : Time Tag received terminal emulator messages on CRLF\r\n");
  printf("       [-V] : Verbose messages\r\n");
  printf("       [-v] : MEGALOADER version number\r\n");
//   printf ("       [-s] : save serial configuration file (you can attach it to problem-question if you have problems)\r\n";
  printf("   [speedword] optional\r\n");
  printf("       [-r 19200, 38400, 57600, 115200] optional - \r\n");
  printf
    ("                 (use after -t) download baud rate (default 19200)\r\n");
  printf
    ("       [-b rate]   optional - terminal emulator baud rate (default 9600)\r\n");
  printf("   [port] optional\r\n");
  printf("       [-c COM1], [-c /dev/ttyS0]\r\n\r\n");
}



/*  Begin Program  */
int
main(int argc, char **argv)
{
  char *fname, *logfname;
  unsigned int i;
  MEMIDX_T ndx;

/************************************************************************
 CODE
 ***********************************************************************/

  scrn_init();
#ifdef __linux__
  clrscr();
#endif


//    unsigned int start, finish;
//    unsigned int data_val;

  /* allocate memory for the far pointer */
  mem_buf = (P_MEM) malloc(sizeof(*mem_buf) * (MEM_MAX + 1));   // 0xffff 16 bit words


  for (ndx = 0; ndx <= MEM_MAX; ndx++)
    mem_buf[ndx] = 0xffff;

  skip_download = 0;
  skip_hexload = 1;
  skip_eepload = 1;

  upload_c = 0;
  upload_t = 0;

  if (argc == 1)                // i.e. pointing to PULOAD
  {
    skip_download = 1;          // just do terminal emulation
  }

  strcpy(sCommPortName, default_comport);       // default com port
  do_tty = 1;
  baud_rate = 9600;
  fname = NULL;

  if (argc > 1) {
    argv++;                     // point to next arg
    argc--;
    strcpy(sArgName, argv[0]);
    if (sArgName[0] != '-') {
      ldr_file = *argv;         // save hex file name
      strcpy(ldr_filename, ldr_file);
      skip_hexload = 0;
      argv++;                   // point to next arg
      argc--;
    }
  }                             // end of if (argc > 1)

  while (argc > 0) {
    strcpy(sArgName, argv[0]);
    if (sArgName[0] == '-') {
      for (i = 1; i < strlen(sArgName); i++) {
        switch (sArgName[i]) {
        case 'h':
          usage();
          exit(0);
          break;
        case 'w':
          write_flash = 1;
          write_marker = 1;
          break;
        case 't':
          talk = 1;
          break;
        case 'E':
          write_marker = 1;
          break;
        case 'T':
          time_tag = 1;
          break;
        case 'V':
          verbose = 1;
          break;
        case 'v':
          sprintf(sArgName,"MEGALOADER verion : 1.1\r\n");
          my_exit(sArgName);
          break;
        case 'b':
          if (argc > 1) {
            argv++;             // point to Com[n] name
            argc--;
            strcpy(sBaudName, argv[0]);
            baud_rate = atoi(sBaudName);
            if (baud_rate == 0) {
              my_exit("Baud Rate - Bad Argument");
            }
          }
          else {
            my_exit("Baud Rate - Bad Argument");
          }
          break;
        case 'r':
          if (argc > 1) {
            argv++;             // point to Com[n] name
            argc--;
            strcpy(sDnBaudName, argv[0]);
            dnload_baud_rate = atoi(sDnBaudName);
            switch (dnload_baud_rate) {
            case 19200:
              break;
            case 38400:
              break;
            case 57600:
              break;
            case 115200:
              break;
            default:
              my_exit("Baud Rate - Bad Argument");
            }
          }
          break;
        case 'c':
          if (argc > 1) {
            argv++;             // point to Com[n] name
            argc--;
            strcpy(sCommPortName, argv[0]);
          }
          else {
            my_exit("Com Port Bad Argument");
          }
          break;
        case 'd':
          if (argc > 1) {
            argv++;             // point to text file name
            argc--;
            fname = *argv;
            cprintf("text file: %s\r\r\n", fname);
          }
          break;
		case 'o':
          if (argc > 1) {
            argv++;             // point to config file name
            argc--;
            fname = *argv;
            cprintf("config file: %s\r\r\n", fname);
			upload_c = 1;
          }
          break;
		case 'a':
          if (argc > 1) {
            argv++;             // point to tables file name
            argc--;
            fname = *argv;
            cprintf("tables file: %s\r\r\n", fname);
			upload_t = 1;
          }
          break;
        case 'e':
          if (argc > 1) {
            argv++;             // point to hex file name
            argc--;
            ldr_file = *argv;
            strcpy(eep_filename, ldr_file);
            cprintf("hex file: %s\r\r\n", eep_filename);
            skip_eepload = 0;
            eep_rom = 0;
          }
          break;
        case 'l':
          if (argc > 1) {
            argv++;             // point to Com[n] name
            argc--;
            logfname = *argv;
            if ((outfile = fopen(logfname, "w")) == NULL) {
              my_exit("Cannot open log file.");
            }
            log_file_open = 1;
          }
          else {
            my_exit("File Name Bad Argument");
          }
          break;
        default:
          my_exit("Bad Argument");
          break;
        } // end of switch
      }  // end of  for (i=1; i < strlen (sArgName);i++)
    }   // end of if (sArgName[0] == '-')
    argv++;                     // point to next arg
    argc--;
  }   // end of while (argc > 1)


//  hStdin = GetStdHandle(STD_INPUT_HANDLE);

#ifdef __linux__
  clrscr();
#endif
  if (OpenSerialPort(sCommPortName) <= 0) {
    sprintf(sArgName,"Unable to open COM port : %s\r\n", sCommPortName);
    my_exit(sArgName);
  }
  if (SetBaudRate((DWORD) baud_rate) <= 0) {
    my_exit("Unable to set 9600 baud");
  }


  EmptyInput(10);
  if ((!skip_hexload) && (ldr_filename != NULL)) {
    hex_load(ldr_filename, ROM_FLASH);
  }

  if ((!skip_eepload) && (eep_filename != NULL)) {
    hex_load(eep_filename, ROM_EEPROM);
  }

  SetBaudRate((DWORD) baud_rate);

  if ((skip_download) || (fname == NULL)) {
    tty();
  }
  else {
	  if((!upload_c) && (!upload_t)) { 
			down_load(fname);
	  }
	  else {
		  if(upload_c) {    
			  upload_config(fname);
		  }
		  else {
			  upload_tables(fname);
		  }

	  }
  }

  // never get here.........
  CleanUp();
#ifdef __linux__
  gotoxy(1, 24);
  clreol();
#endif
  CloseConsole();
  printf("exiting....");
  exit(0);
  return 0;

}                               // END OF MAIN

int
get_prog_type(unsigned char type, char *buf)
{
  int mega_type;
  switch (type) {
  case 0x10:
    strcpy(buf, "AT90S1200 rev. A");
    mega_type = 0;
    break;
  case 0x11:
    strcpy(buf, "AT90S1200 rev. B");
    mega_type = 0;
    break;
  case 0x12:
    strcpy(buf, "AT90S1200 rev. C");
    mega_type = 0;
    break;
  case 0x13:
    strcpy(buf, "AT90S1200       ");
    mega_type = 0;
    break;
  case 0x20:
    strcpy(buf, "AT90S2313       ");
    mega_type = 0;
    break;
  case 0x28:
    strcpy(buf, "AT90S4414       ");
    mega_type = 0;
    break;
  case 0x30:
    strcpy(buf, "AT90S4433       ");
    mega_type = 0;
    break;
  case 0x34:
    strcpy(buf, "AT90S2333       ");
    mega_type = 0;
    break;
  case 0x38:
    strcpy(buf, "AT90S8515       ");
    mega_type = 0;
    break;
  case 0x3A:
    strcpy(buf, "ATmega8515      ");
    mega_type = 8515;
    break;
  case 0x3B:
    strcpy(buf, "ATmega8515 BOOT ");
    mega_type = 8515;
    break;
  case 0x41:
    strcpy(buf, "ATmega103       ");
    mega_type = 0;
    break;
  case 0x42:
    strcpy(buf, "ATmega603       ");
    mega_type = 0;
    break;
  case 0x43:
    strcpy(buf, "ATmega128       ");
    mega_type = 128;
    break;
  case 0x44:
    strcpy(buf, "ATmega128 BOOT  ");
    mega_type = 128;
    break;
  case 0x48:
    strcpy(buf, "AT90S2323       ");
    mega_type = 0;
    break;
  case 0x4C:
    strcpy(buf, "AT90S2343       ");
    mega_type = 0;
    break;
  case 0x50:
    strcpy(buf, "ATtiny11        ");
    mega_type = 0;
    break;
  case 0x51:
    strcpy(buf, "ATtiny10        ");
    mega_type = 0;
    break;
  case 0x55:
    strcpy(buf, "ATtiny12        ");
    mega_type = 0;
    break;
  case 0x56:
    strcpy(buf, "ATtiny15        ");
    mega_type = 0;
    break;
  case 0x58:
    strcpy(buf, "ATtiny19        ");
    mega_type = 0;
    break;
  case 0x5C:
    strcpy(buf, "ATtiny28        ");
    mega_type = 0;
    break;
  case 0x5E:
    strcpy(buf, "ATtiny26        ");
    mega_type = 0;
    break;
  case 0x60:
    strcpy(buf, "ATmega161       ");
    mega_type = 161;
    break;
  case 0x61:
    strcpy(buf, "ATmega161 BOOT  ");
    mega_type = 161;
    break;
  case 0x64:
    strcpy(buf, "ATmega163       ");
    mega_type = 163;
    break;
  case 0x65:
    strcpy(buf, "ATmega83        ");
    mega_type = 83;
    break;
  case 0x66:
    strcpy(buf, "ATmega163 BOOT  ");
    mega_type = 163;
    break;
  case 0x67:
    strcpy(buf, "ATmega83 BOOT   ");
    mega_type = 83;
    break;
  case 0x68:
    strcpy(buf, "AT90S8535       ");
    mega_type = 0;
    break;
  case 0x6A:
    strcpy(buf, "ATmega8535??    ");
    mega_type = 8535;
    break;
  case 0x6B:
    strcpy(buf, "ATmega8535 BOOT?");
    mega_type = 8535;
    break;
  case 0x6C:
    strcpy(buf, "AT90S4434       ");
    mega_type = 0;
    break;
  case 0x70:
    strcpy(buf, "AT90C8534       ");
    mega_type = 0;
    break;
  case 0x71:
    strcpy(buf, "AT90C8544       ");
    mega_type = 0;
    break;
  case 0x72:
    strcpy(buf, "ATmega32        ");
    mega_type = 32;
    break;
  case 0x73:
    strcpy(buf, "ATmega32 BOOT   ");
    mega_type = 32;
    break;
  case 0x74:
    strcpy(buf, "ATmega16        ");
    mega_type = 16;
    break;
  case 0x75:
    strcpy(buf, "ATmega16 BOOT   ");
    mega_type = 16;
    break;
  case 0x76:
    strcpy(buf, "ATmega8         ");
    mega_type = 8;
    break;
  case 0x77:
    strcpy(buf, "ATmega8 BOOT    ");
    mega_type = 8;
    break;
  case 0x79:
    strcpy(buf, "ATmega169 BOOT  ");
    mega_type = 169;
    break;
  case 0x80:
    strcpy(buf, "AT89C1051       ");
    mega_type = 0;
    break;
  case 0x81:
    strcpy(buf, "AT89C2051       ");
    mega_type = 0;
    break;
  case 0x86:
    strcpy(buf, "AT89S8252       ");
    mega_type = 0;
    break;
  case 0x87:
    strcpy(buf, "AT89S53         ");
    mega_type = 0;
    break;
  default:
    strcpy(buf, "Unknown         ");
    mega_type = 0;
    break;
  }
  return mega_type;
}



/**************************************************************************
**
** Function Name: upload_config
**
** Purpose:
**
**  This routine downloads config from AVR mega microprocessor.
**
**
** Inputs:
**
** Outputs:
**
** Functions Activated:
**
** Data Definitions:
**
**
********************************* Function Body *******************************
*/
void
upload_config(char *dnfile)
{
<<<<<<< .mine
  int no_hello;
  int chunkSize;
  char buf[128]; 
  int wait_cnt = 0;
=======
   int no_hello;
   char buf[80];
>>>>>>> .r239

  // try to open the output file for writing
  EmptyInput(10);
  if ((outfile = fopen(dnfile, "w")) == NULL) {
    my_exit("Cannot open config file for writing.");
  }
  
  // succes, contact AVR
  send_message("Man");
  if (verbose)
    cprintf("saying Man at 9600\r\n");
  no_hello = 1;
  if (read_message(6, buf) == 6) 
  {
    if (strstr(buf, "Hello") != NULL) {
      cprintf("AVR said Hello : %s\r\n", buf);
      no_hello = 0;
    }
  }

  if (no_hello) {
      my_exit("No hello from AVR\r\n");
  }
  send_message("mcd");

  while (wait_cnt < serial_timeout) // < 200 ms
  {
    my_delay(1);
    wait_cnt++;
    if (ChkSerialInput() > 0) 
    {
      my_delay(2);
      wait_cnt = 0;  
      memset(buf, 0x00, 128);
      chunkSize = ReadSerialBuff(buf, 16);
      if(chunkSize!=0)
      {
        cprintf("%s", buf);
        fprintf(outfile, "%s", buf);
      }
    }
  }

 fclose(outfile);
 send_message("bye");
 EmptyInput(5);
 
  return;

}


/**************************************************************************
**
** Function Name: upload_tables
**
** Purpose:
**
**  This routine downloads config from AVR mega microprocessor.
**
**
** Inputs:
**
** Outputs:
**
** Functions Activated:
**
** Data Definitions:
**
**
********************************* Function Body *******************************
*/
void
upload_tables(char *dnfile)
{
   int no_hello;
   char buf[80];

  // try to open the output file for writing

  EmptyInput(10);
  if ((outfile = fopen(dnfile, "w")) == NULL) {
     my_exit("Cannot open tables file for writing.");
  }
  
  // succes, contact AVR

 send_message("Man");
 if (verbose)
   cprintf("saying Man at 9600\r\n");
 no_hello = 1;
 if (read_message(6, buf) == 6) {
   if (strstr(buf, "Hello") != NULL) {
     cprintf("AVR said Hello : %s\r\n", buf);
     no_hello = 0;
   }
 }
 if (no_hello) {
    my_exit("No hello from AVR\r\n");
 }
 send_message("mct");

 while(read_message(1, &buf[0]) == 1) {
	if(buf[0] == '\r') {
		buf[0] = '\n';
		EmptyInput(1);
	}
	fprintf(outfile, "%c", buf[0]);
	 
 }

 fclose(outfile);

 send_message("bye");
 EmptyInput(5);
 
  return;

}


/**************************************************************************
**
** Function Name: down_load
**
** Purpose:
**
**  This routine loads text files into the AVR mega microprocessor.
**
**
** Inputs:
**
** Outputs:
**
** Functions Activated:
**
** Data Definitions:
**
**
********************************* Function Body *******************************
*/
void
down_load(char *dnfile)
{

//  DumpInput(10);


  EmptyInput(10);
  strcpy(ldr_filename, dnfile);

  /* *******************  SEND FILE TO PU ******************* */
  cprintf("\r\nsend text file = %s  ", ldr_filename);
  send_file();                  /* com1 = rx file  */

  tty();
  return;

}


/**************************************************************************
**
** Function Name: send_message
**
** Purpose:
**
**  This routine sends  a string to the AVR mega microprocessor comm port.
**
**
** Inputs:
**    *s - character string to send
**
** Outputs:
**
** Functions Activated:
**    WriteSerialChar  ();
**
** Data Definitions:
**
**
********************************* Function Body *******************************
*/
void
send_message(char *s)
/*
 * sends string "s" characters to serial port send char routine
 */
{
  while (*s)
    WriteSerialChar(*s++);
}


/**************************************************************************
**
** Function Name: read_message
**
** Purpose:
**
**  This routine reads a string from the AVR mega microprocessor comm port.
**
**
** Inputs:
**    cnt - # of chars to read
**
** Outputs:
**    s = string read
**
** Functions Activated:
**    WriteSerialChar  ();
**
** Data Definitions:
**
**
********************************* Function Body *******************************
*/
int
read_message(int cnt, char *buf)
/*
 * reads string "s" characters from serial port
 */
{
  unsigned char ichar[1];

  int wait_cnt = 0;
  int read_cnt = 0;
  while (wait_cnt < serial_timeout) {   // < 200 ms
    my_delay(1);
    wait_cnt++;
    if (ChkSerialInput() > 0) {
      ReadSerialChar(ichar);
      buf[read_cnt++] = ichar[0];
      if (read_cnt >= cnt) {
        buf[read_cnt] = 0;
        return read_cnt;
      }
    }
  }
  buf[read_cnt] = 0;
  return read_cnt;
}

int
srl_wait(int cnt, char *buf)
/*
 * reads string "s" characters from serial port
 * waits cnt seconds for data from serial port
 */
{
  int i;
  i = read_message(cnt, buf);
  if (i > 0) {
    if (verbose)
      cprintf(" %c\r\n", buf[0]);
  }
  if (i == 0) {
    cprintf("nothing\r\n");
    buf[0] = 0;
  }
  if (cnt > 0)
    return i;
//  if (buf[0] == '\r')
//    return NULL;
  return 1;
}

/**************************************************************************
**
** Function Name: hex_load
**
** Purpose:
**
**  This routine loads hex files into  AVR mega microprocessor.
**
** Description:
**
** Inputs:
**
** Outputs:
**
** Functions Activated:
**
** Data Definitions:
**
**
********************************* Function Body *******************************
*/
void
hex_load(char *dnfile, int mode)
{
//  int do_load = 0;
  int i, j, insync;
//  int baud_flip;
//  unsigned char ichar[1];
  char buf[40];
  int prog_type;
  int no_hello;


  if (mode == ROM_FLASH)
    cprintf("Down loading flash file:%s\r\n", dnfile);
  if (mode == ROM_EEPROM)
    cprintf("Down loading eeprom file:%s\r\n", dnfile);

  strcpy(ldr_filename, dnfile);
  if (read_rom_file(mode, ldr_filename)) {

    if (talk) {
      sprintf(buf, "%2x", (int) (1000000L / dnload_baud_rate - 0.5));
      send_message("Mancb");
      send_message(buf);
      send_message("cb");
      if (verbose)
        cprintf("saying Mancb%scb at 9600\r\n", buf);

      no_hello = 1;
      if (read_message(6, buf) == 6) {
        if (strstr(buf, "Hello") != NULL) {
          cprintf("AVR said Hello : %s\r\n", buf);
          no_hello = 0;
        }
      }
      if (no_hello) {
        my_exit("No hello from AVR\r\n");
      }
    }
    if (verbose)
      cprintf("Set download baud rate to : %u\r\n", dnload_baud_rate);
    SetBaudRate((DWORD) dnload_baud_rate);

    // now send a p and keep receiving until we get a S
    if (verbose)
      cprintf("we say p\r\n");
    EmptyInput(10);
    WriteSerialChar('p');       // Request sync identifier
    insync = 0;
    for (i = 0; i < 5; i++) {
      j = srl_wait(1, buf);
      if (verbose)
        cprintf("srl cnt = %u buf[0] = %c\r\n", j, buf[0]);
      if (buf[0] == 'S') {
        insync = 1;
        break;
      }
    }
    if (!insync) {
      my_exit("didn't get reply from AVR, out of sync");
    }



    if (verbose)
      cprintf("we say S\r\n");
    EmptyInput(10);
    WriteSerialChar('S');       // Request software identifier
    srl_wait(7, buf);
    if (strstr(buf, "AVR") != buf) {
      sprintf(sArgName,"Sorry, device does not reply AVR... to S, instead:%s\r\n", buf);
      my_exit(sArgName);
    }

    if (verbose)
      cprintf("we say v\r\n");     // Request Software Version
    EmptyInput(10);
    WriteSerialChar('v');
    srl_wait(2, buf);

    if (verbose)
      cprintf("we say V\r\n");     // Request Software Version
    EmptyInput(10);
    WriteSerialChar('V');
    srl_wait(2, buf);

#define DEVTYPE             (0x43)      // mega128 supported device code for avrprog
    if (verbose)
      cprintf("we say t\r\n");     // Request Mega Type Code
    EmptyInput(10);
    WriteSerialChar('t');
    srl_wait(2, buf);
    prog_type = get_prog_type(buf[0], buf);
    if (prog_type != 128) {
      printf(sArgName,"Sorry, device is not Mega128, instead:%s\r\n", buf);
      my_exit(sArgName);
    }
    else {
      if (verbose)
        cprintf("device is Mega128\r\n");
    }

    if (mode == ROM_EEPROM) {
      goto skip_flash;
    }

    if (write_flash) {
      if (verbose)
        cprintf("we are writing too, not only verify! Now say e (erase)\r\n");
      WriteSerialChar('e');     //Chip erase
      serial_timeout = SEC_4_DELAY;
      buf[0] = 0;
      srl_wait(1, buf);
      if (buf[0] == '\r')
        cprintf("Flash erased\r\n");
      else {
        my_exit("Sorry, device is not erased");
      }
      serial_timeout = SERIAL_DELAY;
      burn_flash(prog_type, 0);
      did_flash = 1;
      cprintf("\r\nFlash Download finished\r\n");
    }

    // now verify flash
    cprintf("Verify Flash.\r\n");
    burn_flash(prog_type, 1);


    if (write_marker) {
      if (verbose)
        cprintf("we say E\r\n");
      WriteSerialChar('E');
      srl_wait(1, buf);
    }

  skip_flash:
	;
//    if (mode == ROM_EEPROM) hex_prog(ROM_EEPROM);
//      burn_eeprom(mode,did_flash);

  }

  return;

}






void
send_command(char *buf)
{
  while (*buf != 0)
    WriteSerialChar((unsigned char) *(buf++));
}






/**************************************************************************
**
** Function Name: rs232_out
**
** Purpose:
**     sends a byte out the rs-232 port. a returned value of true is
**     returned if a timeout occurs. a timeout will occur if there is no
**     dsr cts or empty transmit holding register after 65535 attempts.
**
** Description:
**
** Inputs:
**
** Outputs:
**
** Functions Activated:
**
** Data Definitions:
**
**
********************************* Function Body *******************************
*/
int
rs232_out(unsigned char param)
/* sends a byte out the rs-232 port. */
{
  if (WriteSerialChar(param))
    my_exit("Serial Write failure");
  return 0;
}

void
EmptyInput(int count)
{
  int i;
  unsigned char ichar[1];

  for (i = 0; i < count; i++) {
    if (ChkSerialInput() > 0) {
      ReadSerialChar(ichar);
      //disp_ch (ichar[0]);
    }
#ifdef __WIN32__
      my_delay(1);
#endif
  }
}

void
DumpInput(int count)
{
  int i;
  unsigned char ichar[1];

  for (i = 0; i < count; i++) {
    ReadSerialChar(ichar);
    my_delay(1);
  }
}



/**************************************************************************
**
** Function Name: send_file
**
** Purpose:
**     routine to send file to com port
**
** Description:
**
** Inputs:   
**
** Outputs:
**
** Functions Activated:
**
** Data Definitions:
**
**
********************************* Function Body *******************************
*/
int
send_file(void)
{

/* routine to send file to com port */

#include <stdio.h>

  FILE *in;
  long char_cnt;
  unsigned char ch;
  //char windmill[8] = { '/', '-', 0x5c, '|', '/', '-', 0x5c, '|' };
  //int i;

  //i = 0;
  if ((in = fopen(ldr_filename, "rb")) == NULL) {
    cprintf("Cannot open input file.\r\r\n");
    get_filename();
    if ((in = fopen(ldr_filename, "rb")) == NULL) {
      my_exit("Cannot open input file.");
    }
  }

  char_cnt = 0;
//  SetBaudRate((DWORD)57600);
  while (!feof(in)) {
    ch = (unsigned char) fgetc(in);
    if (feof(in))
      break;

    WriteSerialChar((unsigned char) ch);

    char_cnt++;
    if ((char_cnt % 1000) == 0) {
//        i++;
//        if (i > 7)
//          i = 0;
//        printf ("\b%c", windmill[i]);
      cprintf(".");
    }
  }
#if 0
  if (WaitTxBufferEmpty()) {
    my_exit("\r*** WaitTxBufferEmpty Error *** ");
  }
#endif
//  SetBaudRate (baud_rate);
  cprintf("\r\n\r*** %li Chars Sent *** \r\r\n", char_cnt);
  EmptyInput(20);


  fclose(in);
  return 0;
}



/**************************************************************************
**
** Function Name: get_filename
**
** Purpose:
**     enter filename from keyboard
**
** Description:
**
** Inputs:
**
** Outputs:
**
** Functions Activated:
**
** Data Definitions:
**
**
********************************* Function Body *******************************
*/
int
get_filename(void)
{
  cprintf("\r\r\n\r\nPlease enter an file name : ");
  scanf("%30s", ldr_filename);
  return 0;
}





int
disp_ch(int ichar)
{
  static int last_char;
  int hour,min,sec,msec;


  if (hex_mode == HEX_ONLY) {
    cprintf(" %02x", ichar);
    if (log_file_open)
      fprintf(outfile, " %02x", ichar);
  }
  if (hex_mode == ASCII_HEX) {
    if (((ichar > 0x1f) && (ichar < 0x7f)) | (ichar == 0x0d) |
        (ichar == 0x0a)) {
      if ((tag_next) && (time_tag)) {
        My_Get_Time( &hour, &min, &sec, &msec);
        cprintf("%2d:%02d:%02d.%02d =",hour,min,sec,msec);
        if (log_file_open) {
          fprintf(outfile, "%2d:%02d:%02d.%02d =",hour,min,sec,msec);
        }
        tag_next = 0;
      }

      cprintf("%c", ichar);
      if (log_file_open) {
        if (ichar == 0x0a)
          fprintf(outfile, "\r\n");
        if ((ichar != 0x0a) && (ichar != 0x0d))
          fprintf(outfile, "%c", ichar);
      }
      if (ichar == 0x0a)
        tag_next = 1;
    }
    else {
      cprintf(" <%02x>", ichar);
      if (log_file_open)
        fprintf(outfile, " <%02x>\r\n", ichar);
    }
  }
  if (hex_mode == VT_MODE) {
    if (((ichar > 0x1f) && (ichar < 0x7f)) | (ichar == 0x0d) |
        (ichar == 0x0a))
      // a printable character or end of line
    {
      if (time_tag) {
        if (ichar == 0x0d)
          if (last_char != 0x0a) {
            cprintf("\r\n");
            if (log_file_open)
              fprintf(outfile, "\r\n");
          }
        //   if (ichar == 0x0a)
        //     if ( last_char != 0x0d)  printf("\r\n");
        if ((tag_next) && ((ichar > 0x1f) && (ichar < 0x7f))) {
          My_Get_Time( &hour, &min, &sec, &msec);
          cprintf("%2d:%02d:%02d.%02d =",hour,min,sec,msec);
          if (log_file_open) {
            fprintf(outfile, "%2d:%02d:%02d.%02d =",hour,min,sec,msec);
          }
          tag_next = 0;
        }
      }
      if (log_file_open) {
        if (ichar == 0x0a)
          fprintf(outfile, "\r\n");
        if ((ichar != 0x0a) && (ichar != 0x0d))
          fprintf(outfile, "%c", ichar);
      }
      if ((ichar == 0x0d) | (ichar == 0x0a))
        tag_next = 1;
    }
    else {
      if (log_file_open)
        fprintf(outfile, " <%02x>\r\n", ichar);
    }
    vtdecode((unsigned char) ichar);
  }
  last_char = ichar;
  return 0;
}


int
tty(void)
{

#define MAX_STR_LEN 64+1
#define BUF_MAX 512
  int char_in;
  unsigned char ichar[1];
  cprintf("Start terminal emulation - ESC to exit\r\n");
  //refresh();

  char_in = 0;
  hex_mode = VT_MODE;
  int temp;
  
  while (char_in != ENDCHAR) {

#if 1
    while (1) {
      if (ChkSerialInput() > 0) {
        ReadSerialChar(ichar);
        disp_ch(ichar[0]);       // cprintf calls refresh
      }
      else {
        my_delay(1);
        break;
      }
    }
    refresh();
#endif

#ifdef __WIN32__
    if (kbhit()) {
      /* function 0 returns the key that is waiting */
      char_in = my_getch();
#endif

#ifdef __linux__
    if ((char_in = getc(stdin)) != EOF) {
      //  printf(" char typed = %x\r\n",char_in);
#endif
      refresh();

      switch (char_in) {
      case ENDCHAR:
        if (log_file_open)
          fclose(outfile);
        CleanUp ();
        CloseConsole();
        printf("\r\nControl-C Exit\r\n");
        exit (0);

        break;
      case ESCAPE:
        if (log_file_open)
          fclose(outfile);
        CleanUp ();
        CloseConsole();
        printf("\r\nNormal Exit\r\n");
        exit (0);
        break;
      case HEXCHAR:
        //hex_mode = ++hex_mode % 3;
	temp = (++hex_mode) % 3;
	hex_mode = temp;
        if (hex_mode == VT_MODE)
          cprintf("\r\n hex mode = VT_MODE\r\n");
        if (hex_mode == ASCII_HEX)
          cprintf("\r\n hex mode = ASCII_HEX\r\n");
        if (hex_mode == HEX_ONLY)
          cprintf("\r\n hex mode = HEX_ONLY\r\n");
        break;
      case CLEARCHAR:
#ifdef __linux__
		  clrscr();
#endif
        break;
      case LOADRCHAR:
        send_file();
        break;
      case GETNAME:
        get_filename();
        break;
      case CONTROL_R:
        //cprintf("\r\n do RESET\r\n");
        SetDtr(1);
        my_delay(50);
        SetDtr(0);
        break;
      default:
        //cprintf(" send = %x\r\n",char_in);
        rs232_out((unsigned char) char_in);

      }
    }                           /* if kbhit() */
  }                             /* while char_in <> ENDCHAR */
  return 1;
}


